@@ -28,15 +28,15 @@
 #define OPAL_MAJOR_VERSION 4
 #define OPAL_MINOR_VERSION 1
 #define OPAL_RELEASE_VERSION 7
-#define OPAL_GREEK_VERSION "rc1"
+#define OPAL_GREEK_VERSION "a1"
 #define OPAL_WANT_REPO_REV @OPAL_WANT_REPO_REV@
-#define OPAL_REPO_REV "v4.1.5-176-g6d9519e4c3"
+#define OPAL_REPO_REV "v4.1.5-115-g41ba5192d2"
 #ifdef OPAL_VERSION
 /* If we included version.h, we want the real version, not the
    stripped (no-r number) verstion */
 #undef OPAL_VERSION
 #endif
-#define OPAL_VERSION "4.1.7rc1"
-#define OPAL_CONFIGURE_CLI " \'CC=gcc\' \'CXX=g++\' \'FC=nvfortran\' \'LDFLAGS=-Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/25.5/hpcx-12/274637-rel-1/comm_libs/12.8/hpcx/hpcx-2.22.1/ucx/lib -Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/25.5/hpcx-12/274637-rel-1/comm_libs/12.8/hpcx/hpcx-2.22.1/hcoll/lib\' \'--with-platform=../contrib/platform/nvhpc/optimized\' \'--enable-mpi1-compatibility\' \'--with-libevent=internal\' \'--without-xpmem\' \'--without-cray-xpmem\' \'--with-slurm\' \'--with-cuda=/proj/cuda/12.6/Linux_x86_64\' \'--with-hcoll=/proj/nv/libraries/Linux_x86_64/25.5/hpcx-12/274637-rel-1/comm_libs/12.8/hpcx/hpcx-2.22.1/hcoll\' \'--with-ucc=/proj/nv/libraries/Linux_x86_64/25.5/hpcx-12/274637-rel-1/comm_libs/12.8/hpcx/hpcx-2.22.1/ucc\' \'--with-ucx=/proj/nv/libraries/Linux_x86_64/25.5/hpcx-12/274637-rel-1/comm_libs/12.8/hpcx/hpcx-2.22.1/ucx\' \'--prefix=/proj/nv/libraries/Linux_x86_64/25.5/hpcx-12/274637-rel-1/comm_libs/12.8/hpcx/hpcx-2.22.1/ompi\'"
+#define OPAL_VERSION "4.1.7a1"
+#define OPAL_CONFIGURE_CLI " \'CC=gcc\' \'CXX=g++\' \'FC=nvfortran\' \'LDFLAGS=-Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/25.7/hpcx-12/278926-rel-2/comm_libs/12.9/hpcx/hpcx-2.20/ucx/lib -Wl,-rpath-link=/proj/nv/libraries/Linux_x86_64/25.7/hpcx-12/278926-rel-2/comm_libs/12.9/hpcx/hpcx-2.20/hcoll/lib\' \'--with-platform=../contrib/platform/nvhpc/optimized\' \'--enable-mpi1-compatibility\' \'--with-libevent=internal\' \'--without-xpmem\' \'--without-cray-xpmem\' \'--with-slurm\' \'--with-cuda=/proj/cuda/12.5/Linux_x86_64\' \'--with-hcoll=/proj/nv/libraries/Linux_x86_64/25.7/hpcx-12/278926-rel-2/comm_libs/12.9/hpcx/hpcx-2.20/hcoll\' \'--with-ucc=/proj/nv/libraries/Linux_x86_64/25.7/hpcx-12/278926-rel-2/comm_libs/12.9/hpcx/hpcx-2.20/ucc\' \'--with-ucx=/proj/nv/libraries/Linux_x86_64/25.7/hpcx-12/278926-rel-2/comm_libs/12.9/hpcx/hpcx-2.20/ucx\' \'--prefix=/proj/nv/libraries/Linux_x86_64/25.7/hpcx-12/278926-rel-2/comm_libs/12.9/hpcx/hpcx-2.20/ompi\'"
 
 #endif
