@@ -1,6 +1,5 @@
 /*
- * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2004-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2004-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  *
  * This software is available to you under the terms of the
  * OpenIB.org BSD license included below:
@@ -32,9 +31,7 @@
 #ifndef AGG_NODE_H_
 #define AGG_NODE_H_
 
-#include <string.h>
 #include <chrono>
-#include <unordered_map>
 #include <unordered_set>
 #include "agg_types.h"
 #include "smx/smx_types.h"
@@ -97,9 +94,9 @@ class AggPath
 {
     AggNode* m_node_[AGG_PATH_DIRECTION_SIZE];
     uint8_t m_plane_;   // used by FabricGraph for XDR-CR switches (single CR node has several planes)
-    bool m_is_agg_path_valid_;
-    uint8_t m_sw_hops_;        // hops between AN switches
-    Epoch m_agg_path_epoch_;   // last update number
+    bool m_is_valid_;
+    uint8_t m_sw_hops_;   // hops between AN switches
+    Epoch m_epoch_;       // last update number
     AnToAnInfo m_an_to_an_info_;
 
     sharp_path_id_t m_path_id_;
@@ -108,7 +105,6 @@ class AggPath
     AggPathActionNeeded m_action_needed_;
     uint8_t m_active_semaphores_;
     uint8_t m_sat_load_;
-    bool m_mc_load_;   // if true, this path is used by multicast tree
     uint8_t m_exclusive_lock_load_;
     uint16_t m_agg_rate_;
     set<sharp_trees_t> m_tree_load_;
@@ -127,14 +123,13 @@ class AggPath
             const AnToAnInfo& an_to_an_info,
             uint8_t plane)
         : m_plane_(plane),
-          m_is_agg_path_valid_(true),
+          m_is_valid_(true),
           m_sw_hops_(0),
-          m_agg_path_epoch_(epoch),
+          m_epoch_(epoch),
           m_an_to_an_info_(an_to_an_info),
           m_action_needed_(AGG_PATH_ACTION_NEEDED_NONE),
           m_active_semaphores_(0),
           m_sat_load_(0),
-          m_mc_load_(false),
           m_exclusive_lock_load_(0),
           m_agg_rate_(0),
           m_max_llt_qps_(0),
@@ -157,15 +152,14 @@ class AggPath
             const AnToAnInfo& an_to_an_info,
             uint8_t plane)
         : m_plane_(plane),
-          m_is_agg_path_valid_(true),
+          m_is_valid_(true),
           m_sw_hops_(sw_hops),
-          m_agg_path_epoch_(epoch),
+          m_epoch_(epoch),
           m_an_to_an_info_(an_to_an_info),
           m_path_id_(path_id),
           m_action_needed_(AGG_PATH_ACTION_NEEDED_NONE),
           m_active_semaphores_(0),
           m_sat_load_(0),
-          m_mc_load_(false),
           m_exclusive_lock_load_(0),
           m_agg_rate_(0),
           m_max_llt_qps_(0),
@@ -184,13 +178,13 @@ class AggPath
 
     sharp_path_id_t GetId() const { return m_path_id_; }
 
-    void SetAggPathValid(bool is_valid, uint64_t epoch)
+    void SetValid(bool is_valid, uint64_t epoch)
     {
-        m_is_agg_path_valid_ = is_valid;
+        m_is_valid_ = is_valid;
         SetEpoch(epoch);
     }
 
-    bool IsExclusiveLockAvailable() const { return (!IsMcLoadSet() && m_sat_load_ < m_active_semaphores_) ? true : false; }
+    bool IsExclusiveLockAvailable() const { return m_sat_load_ < m_active_semaphores_ ? true : false; }
 
     void IncSatLoad() { m_sat_load_++; }
 
@@ -198,17 +192,11 @@ class AggPath
 
     uint32_t GetSatLoad() { return m_sat_load_; }
 
-    void SetMcLoad() { m_mc_load_ = true; }
-
-    void UnSetMcLoad() { m_mc_load_ = false; }
-
-    bool IsMcLoadSet() const { return m_mc_load_; }
-
     void IncExclusiveLockLoad() { m_exclusive_lock_load_++; }
 
     void DecExclusiveLockLoad();
 
-    bool IsLockAvailable() const { return (!IsMcLoadSet() && m_exclusive_lock_load_ < m_active_semaphores_) ? true : false; }
+    bool IsLockAvailable() const { return m_exclusive_lock_load_ < m_active_semaphores_ ? true : false; }
 
     bool HasQPsQuota(uint16_t qps_number, bool is_sat = false)
     {
@@ -243,7 +231,7 @@ class AggPath
     void EraseTreeId(sharp_trees_t tree_id);
     uint8_t GetTreeLoadSize();
 
-    bool IsAggPathValid() const { return m_is_agg_path_valid_; }
+    bool IsValid() const { return m_is_valid_; }
 
     void SetAggRate(uint16_t rate) { m_agg_rate_ = rate; }
 
@@ -266,10 +254,10 @@ class AggPath
     uint8_t GetSwHops() const { return m_sw_hops_; }
     void SetSwHops(uint8_t sw_hops) { m_sw_hops_ = sw_hops; }
 
-    void SetEpoch(uint64_t epoch) { m_agg_path_epoch_.SetEpoch(epoch); }
+    void SetEpoch(uint64_t epoch) { m_epoch_.SetEpoch(epoch); }
 
-    uint64_t GetCurrEpoch() { return m_agg_path_epoch_.m_curr_epoch_; }
-    uint64_t GetPrevEpoch() { return m_agg_path_epoch_.m_prev_epoch_; }
+    uint64_t GetCurrEpoch() { return m_epoch_.m_curr_epoch_; }
+    uint64_t GetPrevEpoch() { return m_epoch_.m_prev_epoch_; }
 
     AggNode* GetRemoteNode(const AggNode* local_agg_node) const
     {
@@ -301,10 +289,10 @@ class AggNode
 
     MapAggNodeKeyToPath m_path_[AGG_PATH_DIRECTION_SIZE];
     SetAggPathPtr m_agg_paths_;
-
-    SwitchIssuState m_issu_;   // Tells whether the AggNode is going through an upgrade process
+    // MapAggNodeKeyToPath m_down_path_;
    private:
-    sharp_an_id_t m_an_id_;
+    // In general size of m_an_ids_ is 1. Only for XDR CR an_ids according to max_plane.
+    std::vector<sharp_an_id_t> m_an_ids_;
     static sharp_an_id_t m_next_id_;
 
     const Port* m_port_;   // containing port in fabric_graph
@@ -317,9 +305,7 @@ class AggNode
     // Indicator that port resources configuration is required for this AggNode
     bool m_configure_port_resources_;
 
-    // Non planarized table are located at m_an_min_hops_table_[0]
-    // Planarized tables are located according to plane index ( index = plane - 1)
-    AnMinHopsTableByPlane m_an_min_hops_table_;
+    AnMinHopsTable m_an_min_hops_table_;
     string m_switch_desc_;
     sharp_resource_priority m_priority_;
 
@@ -331,7 +317,7 @@ class AggNode
             sharp_an_id_t an_id,
             uint64_t am_key,
             string switch_desc,
-            const AnMinHopsTableByPlane& an_min_hops_table);
+            const AnMinHopsTable an_min_hops_table);
 
     virtual ~AggNode();   // Keep the destructor virtual, so that child classes also get their destructor
 
@@ -346,13 +332,15 @@ class AggNode
     uint8_t GetPortRank() const { return m_agg_port_info_.m_port_info.m_peer_topology_info.GetRank(); }
     inline uint8_t GetPortRankIfAvailable() const { return m_agg_port_info_.m_port_info.m_peer_topology_info.GetRankIfAvailable(); }
     coordinate_t GetCoordinatesIfAvailable() const { return m_agg_port_info_.m_port_info.m_peer_topology_info.GetCoordinatesIfAvailable(); }
-    island_id_t GetIslandId() const { return GetCoordinatesIfAvailable(); }
     const string GetDescription() const;
-    sharp_an_id_t GetId() const { return m_an_id_; }
+    sharp_an_id_t GetId(uint8_t plane = 0) const
+    {
+        return (plane && m_an_ids_.size() > 1 && plane <= m_an_ids_.size()) ? m_an_ids_[plane - 1] : m_an_ids_[0];
+    }
+    uint16_t GetIdsSize() { return m_an_ids_.size(); }
     static sharp_an_id_t GetAnNodesNumber() { return m_next_id_; }
     port_key_t GetPortKey() const { return m_agg_port_info_.m_port_info.m_port_key; }
-    inline SwitchIssuState GetIssu() const { return m_issu_; }
-    inline void SetIssu(SwitchIssuState issu) { m_issu_ = issu; }
+    uint8_t GetPlane() const { return m_agg_port_info_.m_port_info.m_port_key.GetPlane(); }
 
     const AggNodeInfo& GetAggNodeInfo() const { return m_agg_node_info_; }
     void SetAggNodeInfo(const AggNodeInfo* p_agg_node_info)
@@ -471,24 +459,14 @@ class AggNode
 
     const MapAggNodeKeyToPath& GetAggPathsByDirection(AggPathDirection direction) const;
 
-    // Fills 'adjacents' with nodes that are adjacent to this and have the same rank.
-    // If 'p_adjacent_to_link_count' is not null, it is filled with number of parallel links per adjacent node.
-    // if 'sat_requsted', allow only nodes that their path can be locked or, if 'lock_exclusively', can be exclusively locked.
-    // Used in DFP topology.
-    void GetAdjacentSiblings(bool sat_requsted,
-                             bool lock_exclusively,
-                             std::unordered_set<const AggNode*>& adjacent,
-                             std::unordered_map<const AggNode*, node_degree_t>* p_adjacent_to_link_count = nullptr) const;
+    // Returns nodes that are adjacent to this and have the same rank. Used in DFP topology.
+    // if sat_requsted, allow only nodes that their path can be locked or, if lock_exclusively, can be exclusively locked.
+    void GetAdjacentSibling(std::unordered_set<const AggNode*>& adjacents, bool sat_requsted, bool lock_exclusively) const;
 
-    AggPath* GetPathMinTreeLoad(AggNode* p_remote_agg_node, sharp_trees_t tree_id, uint8_t plane_number);
-
-    // Get Path to p_remote_agg_node on direction, with zero tree load
-    AggPath* GetUnusedPath(AggNode* p_remote_agg_node, AggPathDirection direction, uint8_t plane_number);
+    AggPath* GetPathMinTreeLoad(AggNode* p_remote_agg_node, sharp_trees_t tree_id);
 
     AggPath* GetPathByPort(AggNode* p_remote_agg_node, sharp_trees_t tree_id, uint32_t port);
 
-    AggPath* GetPathByPort(phys_port_t phys_port, AggPathDirection direction);
-
     void SetPort(Port* p_port) { m_port_ = p_port; }
     const Port* GetPort() const { return m_port_; }
 
@@ -509,11 +487,10 @@ class AggNode
     // Return port resources configuration required indicator
     bool GetConfigurePortResources() const { return m_configure_port_resources_; }
 
-    AnMinHopsTableByPlane& GetAnMinHops() { return m_an_min_hops_table_; };
-    AnMinHopsTable& GetAnMinHops(uint8_t plane);
-    const AnMinHopsTableByPlane& GetAnMinHops() const { return m_an_min_hops_table_; };
+    AnMinHopsTable& GetAnMinHops() { return m_an_min_hops_table_; };
+    const AnMinHopsTable& GetAnMinHops() const { return m_an_min_hops_table_; };
 
-    void UpdateAnMinhopTable(const AnMinHopsTableByPlane& an_min_hops_table)
+    void UpdateAnMinhopTable(const AnMinHopsTable& an_min_hops_table)
     {
         if (an_min_hops_table.size())
             m_an_min_hops_table_ = an_min_hops_table;
@@ -598,7 +575,7 @@ class AggNodeFabricIterator : std::iterator<std::input_iterator_tag, AggNodeFabr
 struct topology_minhop_tables
 {
     std::map<uint64_t, std::list<uint64_t>> adjacent_table;
-    std::map<uint64_t, MinHopsTableByPlane> minhop_tables;
+    std::map<uint64_t, MinHopsTable> minhop_tables;
     // guid to min_hop_key(LID) table
     // should be removed once node ID is implemented
     std::map<uint64_t, node_min_hop_key_t> minhop_keys_table;
@@ -665,7 +642,7 @@ class FabricTopologyData
 
     struct topology_minhop_tables& GetTopologyMinHopTables() { return m_topology_info_.m_topology_minhop_tables; }
     std::map<uint64_t, std::list<uint64_t>>& GetAdjacentTable() { return m_topology_info_.m_topology_minhop_tables.adjacent_table; }
-    std::map<uint64_t, MinHopsTableByPlane>& GetMinHopTables() { return m_topology_info_.m_topology_minhop_tables.minhop_tables; }
+    std::map<uint64_t, MinHopsTable>& GetMinHopTables() { return m_topology_info_.m_topology_minhop_tables.minhop_tables; }
     std::map<uint64_t, node_min_hop_key_t>& GetMinHopKeysTable() { return m_topology_info_.m_topology_minhop_tables.minhop_keys_table; }
 
     void InsertAggNode(AggNode* p_agg_node);
