@@ -1,6 +1,5 @@
 /*
- * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2012-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2012-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -35,7 +34,7 @@
 #ifndef AGG_FABRIC_H_
 #define AGG_FABRIC_H_
 
-#include "common_types.h"
+#include <unordered_map>
 
 #include "agg_node.h"
 #include "agg_types.h"
@@ -44,25 +43,73 @@
 #include "dump_file.h"
 #include "fabric_dump_parser.h"
 #include "fabric_provider.h"
+#include "option_manager.h"
 #include "port_data.h"
 #include "smx/smx_types.h"
-#include "telemetry/telemetry.h"
+#include "telemetry.h"
 #include "thread_pool.h"
 #include "tree_manager.h"
-class SharableMlid;
-using SharableMlidPtr = std::shared_ptr<SharableMlid>;
-using MapAggNodeKeyToMlidList = std::map<uint64_t, ListLids>;
 
-class AggTree;
-class AggTreeDeleter;
+class FabricProvider;
+class TreeManager;
+class CommandManager;
+class TreeNode;
+class TreeEdge;
+class HostInfo;
+class PortData;
+class SharpJob;
+class AggNode;
+class AggNodeFabricInfo;
+struct SpanningInfo;
+struct QPData;
 
-using AggTreeUniquePtr = std::unique_ptr<AggTree, AggTreeDeleter>;
-using MapTreeIdToAggTree = std::map<sharp_trees_t, AggTreeUniquePtr>;
-using MapExtJobIdToMapTreeIdToTree = std::unordered_map<SharpExtJobId, MapTreeIdToAggTree>;
+struct HostInfoSort
+{
+    inline bool operator()(const HostInfo* p_lhs, const HostInfo* p_rhs) const;
+};
 
-// Temporary values that will be replaced by data received from SM
-#define DEFAULT_MIN_MLID 0xC000
-#define DEFAULT_MAX_MLID 0xCFA0   // Range of 4000 mlids
+struct TreeNodePtrSort
+{
+    bool operator()(const TreeNode* lhs, const TreeNode* rhs) const;
+};
+
+typedef std::set<SharpJob*> SetJobPtr;
+typedef std::unordered_set<SharpJob*> UnorderedSetJobPtr;
+typedef std::map<sharp_job_id_t, SetTreeIds> MapSharpJobIdToTreeIds;
+typedef std::set<class TreeNode*> SetTreeNodePtr;
+typedef std::set<class TreeNode*, TreeNodePtrSort> SetTreeNodePtrSortedByANGuid;
+typedef std::set<const HostInfo*, HostInfoSort> SetHostInfoPtr;
+typedef std::set<port_key_t> SetPortKey;
+typedef std::list<class PortData*> ListPortDataPtr;
+typedef std::list<struct SpanningInfo> ListSpanningInfo;
+typedef std::vector<class AggNodeFabricInfo*> VecAnFabricInfoPtr;
+typedef std::vector<class TreeNode*> VecTreeNodePtr;
+typedef std::unordered_map<sharp_trees_t, class TreeNode*> MapTreeIdToTreeNode;
+typedef std::unordered_map<uint64_t, std::string> MapTreeHashToDumpMessage;
+typedef std::vector<class TreeEdge> VecTreeEdge;
+typedef std::vector<class AggTree*> VecAggTreePtr;
+typedef std::map<sharp_trees_t, class AggTree*> MapTreeIdToAggTree;
+typedef std::unordered_map<SharpExtJobId, MapTreeIdToAggTree> MapExtJobIdToMapTreeIdToTree;
+typedef VecAggTreePtr::iterator AggTreesIter;
+typedef VecAggTreePtr::const_iterator AggTreesConstIter;
+
+typedef std::map<string, ListAggNodeFabricPtr> MapStrListAggNodeFabricPtr;
+typedef std::map<string, class AggNodeFabricInfo*> MapStrAggNodeFabricPtr;
+typedef std::multimap<port_key_t, class AggNodeFabricInfo*> MapPortKeyToAggNodeFabrics;
+typedef std::map<string, class HostInfo*> MapStrToHostInfo;
+
+typedef std::map<port_key_t, port_key_t> MapPortKeyToPortKey;
+typedef std::map<uint32_t, QPData> MapQPNToQPData;
+typedef MapQPNToQPData::iterator MapQPNToQPDataIter;
+typedef MapPortKeyToPortKey::iterator MapPortKeyToPortKeyIter;
+typedef std::multimap<port_key_t, port_key_t> MapPortKeyToPortKeys;
+typedef MapPortKeyToPortKeys::iterator MapPortKeyToPortKeysIter;
+
+typedef std::map<std::pair<port_key_t, port_key_t>, uint16_t> MapTreeTurns;
+typedef std::map<port_key_t, AggNodeRecord> MapPortKeyToAggNodeRecord;
+typedef std::map<port_key_t, AggNodeInfoRecord> MapPortKeyToAggNodeInfoRecord;
+typedef std::map<string, uint32_t> MapFileNameToCRC;
+typedef std::vector<struct AggPathRecord> VecAggPathRecord;
 
 enum class DynamicTreeAlgoEnum
 {
@@ -70,17 +117,11 @@ enum class DynamicTreeAlgoEnum
     QUASI_FAT_TREE_ORIENTED_ALGORITHM = 1
 };
 
-class AggTreeDeleter
-{
-   public:
-    void operator()(AggTree* ptr) const;
-};
-
 class HostInfo
 {
     string m_host_name_;
     sharp_sd_info m_sharpd_info_;
-    MapHcaPortDataPtr m_ports_data_;
+    MapPortDataPtr m_ports_data_;
 
     static sharpd_id_t m_next_id_;
 
@@ -88,15 +129,15 @@ class HostInfo
     explicit HostInfo(const string& host_name);
 
     void AddPortData(PortData* p_port_data);
-    void ChangePortDataState(const hca_port_key_t port_key, const bool should_enable);
-    void EnablePortData(const hca_port_key_t port_key);
-    void DisablePortData(const hca_port_key_t port_key);
-    void DeletePortFromPortsData(const hca_port_key_t port_key);
+    void ChangePortDataState(const port_key_t port_key, const bool should_enable);
+    void EnablePortData(const port_key_t port_key);
+    void DisablePortData(const port_key_t port_key);
+    void DeletePortFromPortsData(const port_key_t port_key);
 
     inline const string& GetHostName() const { return m_host_name_; }
     inline const sharp_sd_info& GetSharpdInfo() const { return m_sharpd_info_; }
     inline sharpd_id_t GetSharpdId() const { return m_sharpd_info_.sharpd_id; }
-    inline const MapHcaPortDataPtr& GetPortsData() const { return m_ports_data_; }
+    inline const MapPortDataPtr& GetPortsData() const { return m_ports_data_; }
 };
 
 struct SpanningInfo
@@ -118,21 +159,84 @@ struct SpanningInfo
     bool operator>(SpanningInfo const& rhs) const { return (rhs < *this); }
 };
 
-// the same mlid is shared between all planes
-class SharableMlid
+// use bits values to enable selecting several states (e.g. configured or error)
+enum QpStateEnum
 {
-    lid_t m_mlid_;   // mlid of all-gather multicast tree
+    QP_STATE_UNALLOCATED = 1,
+    QP_STATE_ALLOCATED = 2,
+    QP_STATE_CONFIGURED = 4,
+    QP_STATE_ERROR = 8,
+    QP_STATE_UNALLOCATE_REQUIRED = 16,
+    QP_STATE_UNCONFIGURE_REQUIRED = 32,
+    QP_STATE_UNCONFIGURE = (QP_STATE_UNCONFIGURE_REQUIRED | QP_STATE_CONFIGURED)
+};
+
+static inline const char* QpStateToStr(QpStateEnum qp_state)
+{
+    switch (qp_state) {
+        case QP_STATE_UNALLOCATED:
+            return ("unallocated");
+        case QP_STATE_ALLOCATED:
+            return ("allocated");
+        case QP_STATE_CONFIGURED:
+            return ("configured");
+        case QP_STATE_ERROR:
+            return ("error");
+        case QP_STATE_UNALLOCATE_REQUIRED:
+            return ("unallocate required");
+        case QP_STATE_UNCONFIGURE_REQUIRED:
+            return ("unconfigure required");
+        case QP_STATE_UNCONFIGURE:
+            return ("unconfigure required or configured");
+        default:
+            return ("unknown");
+    }
+};
+
+struct QPData
+{
+    uint32_t qpn;
+    uint32_t index;
+    QpStateEnum state;
+    SharpMtu mtu;
+    lid_t rlid;
+    uint8_t ts;
+    uint32_t rqpn;
+    bool is_parent;
+    sharp_trees_t tree_id;
+    uint32_t port;
+    bool is_root_qp;
+    bool is_multicast_qp;
+};
+
+class Qp
+{
+    uint32_t m_qp_num_;
+    QpStateEnum m_qp_state_;
 
    public:
-    SharableMlid(lid_t mlid) : m_mlid_(mlid) {}
-    ~SharableMlid();
-    lid_t GetMlid() { return m_mlid_; }
+    Qp() : m_qp_num_(0), m_qp_state_(QP_STATE_UNALLOCATED) {}
+
+    void SetNumber(uint32_t qp_number) { m_qp_num_ = qp_number; }
+
+    uint32_t GetNumber() const { return m_qp_num_; }
+
+    QpStateEnum GetState() const { return m_qp_state_; }
+
+    void Set(uint32_t qp_num, QpStateEnum qp_state, AggNodeFabricInfo* p_an_fabric_info);
+
+    void SetState(QpStateEnum qp_state) { m_qp_state_ = qp_state; }
+
+    void Clear()
+    {
+        m_qp_num_ = 0;
+        m_qp_state_ = QP_STATE_UNALLOCATED;
+    }
 };
 
 class Fabric
 {
-    MapHCAPortDataUniquePtr m_hca_port_data_by_key_;
-    MapPortDataUniquePtr m_an_port_data_by_key_;
+    MapPortDataUniquePtr m_port_data_by_key_;
     VecPortDataPtr m_old_ports_;
     AggregatedLogMessage m_disabled_ports_msg_;
     MapStrListAggNodeFabricPtr m_agg_node_by_desc_;         // Nodes list ptr by node desc
@@ -148,33 +252,30 @@ class Fabric
     VecAggPathPtr m_agg_paths_;
 
     ////Optimization
-    MapHcaPortKeyToPortKey m_compute_to_sw_port_;
+    MapPortKeyToPortKey m_compute_to_sw_port_;
     // set of closest AggNode to compute port
     // we might have several AggNodes to compute ports if not directly connected
-    MapHcaPortKeyToAggNodeFabrics m_compute_port_to_agg_nodes_;
+    MapPortKeyToAggNodeFabrics m_compute_port_to_agg_nodes_;
     MapPortKeyToAggNodeFabricPtr m_sw_to_agg_node_;
 
     // set of all computes PortData that are connected to ANs not spans by trees
     SetPortDataConstPtr m_compute_not_on_tree_;
-    MapPortKeyToHCAPortKeys m_sw_with_no_an_to_compute_ports_;
+    MapPortKeyToPortKeys m_sw_with_no_an_to_compute_ports_;
 
     ////collection for the current epoch
-    MapPortKeyToHCAPortKeys m_sw_to_update_compute_ports_;
+    MapPortKeyToPortKeys m_sw_to_update_compute_ports_;
 
     //  accumulates updates until the next build_trees is executed
-    MapHcaPortKeyToAggNodeFabrics m_update_compute_to_agg_nodes_;
+    MapPortKeyToAggNodeFabrics m_update_compute_to_agg_nodes_;
 
     uint64_t m_epoch_;   // Current update index
     bool m_trees_built_;
     uint16_t m_llt_trees_to_build_;
     uint16_t m_max_tree_id_;
     uint16_t m_max_llt_tree_id_;
-    sharp_trees_t m_min_mc_tree_id_;   // first multicast tree id
-    sharp_trees_t m_max_mc_tree_id_;   // last multicast tree id
-    bool m_dynamic_tree_allocation{false};
+    bool m_dynamic_tree_allocation;
     DynamicTreeAlgoEnum m_dynamic_tree_algo;
     port_key_t m_sm_port_guid_;
-    uint64_t m_sa_key_;
 
     FabricProvider* m_fabric_provider_;
     TreeManager m_tree_manager_;
@@ -193,14 +294,7 @@ class Fabric
     MapTreeHashToDumpMessage m_trees_history_by_hash_;
     std::unique_ptr<file_utils::DumpFile> m_trees_history_file_ptr_;
     bool m_initialized_;
-    Metrics m_metrics_{Metrics::Config{}, ""};   // will be initialise in Init()
-
-    // All gather multicat tree mlid managment
-    lid_t m_min_mlid_;            // minimal multicast lid
-    lid_t m_max_mlid_;            // maximal multicast lid
-    SetLids m_available_mlids_;   // Set of available mlids
-
-    MapAggNodeKeyToMlidList m_required_mc_query_;   // multicast table information required by seamless restart
+    Metrics m_metrics_{MetricsConfig{}, ""};   // will be initialise in Init()
 
     void CheckUpdateTreesPerPortSet(const SetPortDataConstPtr& compute_ports);
     int CreateDumpDirectory(const string& dir_name) const;
@@ -217,11 +311,9 @@ class Fabric
           m_llt_trees_to_build_(0),
           m_max_tree_id_(0),
           m_max_llt_tree_id_(0),
-          m_min_mc_tree_id_(0),
-          m_max_mc_tree_id_(0),
+          m_dynamic_tree_allocation(false),
           m_dynamic_tree_algo(DynamicTreeAlgoEnum::SUPER_POD_ORIENTED_ALGORITHM),
           m_sm_port_guid_(),
-          m_sa_key_(0x1),
           m_fabric_provider_(NULL),
           m_tree_manager_(),
           m_command_manager_(NULL),
@@ -229,26 +321,20 @@ class Fabric
           m_agg_nodes_file_update_req_(false),
           m_is_seamless_restart_in_progress_(false),
           m_rebuild_trees_required_(false),
-          m_initialized_(false),
-          m_min_mlid_(DEFAULT_MIN_MLID),
-          m_max_mlid_(DEFAULT_MAX_MLID)
+          m_initialized_(false)
     {}
 
     ~Fabric();
 
-    int Init(int& seamless_restart, TelemetryConfig cfg, std::string host_name);
+    int Init(int& seamless_restart, MetricsConfig cfg, std::string host_name);
 
     void ConfigureLocalPort(port_key_t local_port);
     void CleanFabricProvider();
     void Finish();   // explicit destruction
 
     uint16_t GetMaxTreeId() { return m_max_tree_id_; }
-    uint16_t GetMaxLltTreeId() { return m_max_llt_tree_id_; }
-    sharp_trees_t GetMinMulticastTreeId() const { return m_min_mc_tree_id_; }
-    sharp_trees_t GetMaxMulticastTreeId() const { return m_max_mc_tree_id_; }
 
-    lid_t GetMinMlid() const { return m_min_mlid_; }
-    lid_t GetMaxMlid() const { return m_max_mlid_; }
+    uint16_t GetMaxLltTreeId() { return m_max_llt_tree_id_; }
 
     bool IsDynamicTreeAllocation() const { return m_dynamic_tree_allocation; }
 
@@ -283,7 +369,6 @@ class Fabric
     sharp_trees_t GetLltTreeId(sharp_trees_t sat_tree_id);
 
     bool IsSatTreeId(sharp_trees_t tree_id) const;
-    bool IsMulticastTreeId(sharp_trees_t tree_id) const;
 
     FabricProvider* GetFabricProvider() const { return m_fabric_provider_; }
 
@@ -329,13 +414,10 @@ class Fabric
     void DeleteTree(sharp_trees_t tree_id);
     void DeleteAllTrees();
 
-    AggTree& CreateJobTree(const SharpExtJobId& external_job_id, sharp_trees_t tree_id, uint8_t tree_plane);
-    AggTree& CreateJobMulticastTree(const SharpExtJobId& external_job_id, uint8_t tree_plane, SharableMlidPtr& sharable_mlid);
-
-    bool DeleteJobTrees(const SharpExtJobId& external_job_id, bool force = false, bool clean_flow = false);
+    AggTree* CreateJobTree(const SharpExtJobId& external_job_id, sharp_trees_t tree_id);
+    void DeleteJobTrees(const SharpExtJobId& external_job_id, bool force = false, bool clean_flow = false);
     // delete all job trees in a force=true manner
     void DeleteAllJobTrees();
-    const MapTreeIdToAggTree* GetJobTrees(const SharpExtJobId& external_job_id);
     AggTree* GetJobTree(const SharpExtJobId& external_job_id, sharp_trees_t tree_id);
     void GetJobTreeIds(const SharpExtJobId& external_job_id, SetTreeIds& set_tree_id);
 
@@ -365,19 +447,19 @@ class Fabric
     void CheckUpdateTrees(const SharpJob* p_job);
 
     int FabricUpdateEnd();
-    void StartFabric(uint32_t agg_nodes_number,
-                     uint32_t paths_number,
-                     sharp_job_id_t max_jobs_number,
-                     uint8_t control_path_version,
-                     uint16_t tree_table_size);
-
-    void SetSaKey(uint64_t sa_key) { m_sa_key_ = sa_key; };
+    void Start(uint32_t agg_nodes_number,
+               uint32_t paths_number,
+               sharp_job_id_t max_jobs_number,
+               uint8_t control_path_version,
+               uint16_t tree_table_size);
 
     // get node by desc or node key
     AggNodeFabricInfo* GetAggNode(const string& node_desc, port_key_t port_key);
     AggNodeFabricInfo* GetAggNode(port_key_t port_key);
     AggNodeFabricInfo* GetAggNode(const string& node_desc);
     AggNodeFabricInfo* GetAggNode(lid_t lid);
+    AggNodeFabricInfo* GetAggNode(uint64_t guid);
+    AggNodeFabricInfo* GetAggNode(uint64_t guid, phys_port_t port_number);
 
     AggNodeFabricInfo* GetAggNodeById(sharp_an_id_t an_id) const;
     void InsertAggNodeById(AggNodeFabricInfo* p_agg_node);
@@ -393,23 +475,20 @@ class Fabric
     const HostInfo* GetHostInfo(const string& host_name) const;
     uint8_t GetNumHops(AggNodeFabricInfo* p_agg_node1, AggNodeFabricInfo* p_agg_node2);
 
-    const PortData* FindConstHcaPortData(hca_port_key_t port_key) const;
-    const PortData* FindConstAnPortData(port_key_t port_key) const;
+    const PortData* FindConstPortData(port_key_t port_key) const;
 
     uint8_t GetControlPathVersion() const { return m_control_path_version_; }
 
     int GetMinimalShortestPathGraph(uint64_t* guids,
                                     uint32_t num_guids,
                                     std::map<uint64_t, std::set<uint64_t>>& sub_graph,
-                                    std::map<uint64_t, std::list<uint64_t>>& leaf_to_hosts,
-                                    uint8_t plane);
+                                    std::map<uint64_t, std::list<uint64_t>>& leaf_to_hosts);
 
     void SubDivide(uint64_t root,
                    uint64_t src,
                    std::list<uint64_t>& destinations,
                    std::map<uint64_t, std::set<uint64_t>>& known_dst_map,
-                   std::map<uint64_t, std::set<uint64_t>>& sub_graph,
-                   uint8_t plane);
+                   std::map<uint64_t, std::set<uint64_t>>& sub_graph);
 
     void UpdateAggNodeInfo(AggNodeFabricInfo* p_agg_node, const AggNodeInfo& agg_node_info, const PortInfo& port_info);
 
@@ -426,12 +505,14 @@ class Fabric
     void MarkInvalidSwitchesForRediscovery();
 
     // This method returns the leaf switch that is directly connected to 'port_key, NULL if not found
-    AggNodeFabricInfo* GetSwitchThatIsConnectedToHost(hca_port_key_t port_guid);
+    AggNodeFabricInfo* GetSwitchThatIsConnectedToHost(port_key_t port_guid);
 
-    bool IsReconstructMulticastTrees() { return !m_required_mc_query_.empty(); }
-    void SetRequiredMulticastTableQuery(persistent_job_info* job_info);
-    int ReadRequiredMulticastTables();
-    bool AllocateMlid(lid_t mlid);   // Allocate specific mlid, used by seamless restart
+    int BuildQuasiFatTreeForJob(const SetPortDataConstPtr& compute_ports,
+                                JobSubTreeInfo& job_sub_tree_info,
+                                const JobResource& job_resource,
+                                JobSubTreeScore& sub_tree_score,
+                                TreeNodesVecUniquePtr& tmp_tree_nodes_vec,
+                                const SharpExtJobId& external_job_id);
 
     ////////////////////////////////////////////
     /// Fabric CSV Parser Call Backs Function
@@ -451,16 +532,13 @@ class Fabric
    private:
     // find existing PortData
     // Returns NULL if not found
-    PortData* FindHcaPortData(hca_port_key_t port_key, const bool should_ignore_invalid_ports = true);
-    PortData* FindAnPortData(port_key_t port_key, const bool should_ignore_invalid_ports = true);
-    PortData* CreateHcaPortData(const PortInfo& port_info);
-    PortData* CreateAnPortData(const PortInfo& port_info);
+    PortData* FindPortData(port_key_t port_key, const bool should_ignore_invalid_ports = true);
+    PortData* CreatePortData(const PortInfo& port_info);
     void CreateVportData(const PortInfo& vport_info, const std::string& host_name, const PortInfo& phys_port_info);
 
     void CreatePath(AggNodeFabricInfo* p_down_node,
                     AggNodeFabricInfo* p_up_node,
                     sharp_path_id_t path_id,
-                    uint8_t plane_number,
                     uint8_t sw_hops,
                     uint64_t epoch,
                     uint8_t num_semaphores,
@@ -477,7 +555,7 @@ class Fabric
     void AddAggNode(PortData* p_port_data,
                     const AggNodeInfo& agg_node_info,
                     sharp_an_id_t an_id,
-                    const AnMinHopsTableByPlane& an_min_hops_table,
+                    const AnMinHopsTable& an_min_hops_table,
                     uint64_t am_key,
                     string switch_desc);
     int AddHostInfoToVport(PortData* p_vport_data, const string& host_name);
@@ -488,7 +566,7 @@ class Fabric
     void DeleteAllHosts();
 
     void HandlemComutesOnSwitchWithNewAggNode(port_key_t sw_key);
-    void EraseSwMultimapElement(MapPortKeyToHCAPortKeys& sw_multimap, port_key_t sw_key, hca_port_key_t element);
+    void EraseSwMultimapElement(MapPortKeyToPortKeys& sw_multimap, port_key_t sw_key, port_key_t element);
 
     // Associate compute ports to aggregation nodes
     // This method needs to be invoked before invoking UpdateComputeToTreeNode
@@ -499,7 +577,7 @@ class Fabric
     // UpdateComputeToAggNode needs to be invoked before this method
     int UpdateComputeToTreeNode(bool use_all_computes = false);
     // clear mapping between compute to tree nodes on all valid trees
-    void ClearComputeToTreesNode(hca_port_key_t compute_port_key);
+    void ClearComputeToTreesNode(port_key_t compute_port_key);
 
     // old virtual port is a port that was deleted from m_port_data_by_key_ because fabric received INACTIVE update for it
     void ActivateVport(const PortUpdateInfo& port_update_info);
@@ -534,11 +612,13 @@ class Fabric
     {
         try {
             std::unique_ptr<PortData> p_port_data = std::unique_ptr<PortData>(new PortType(port_info, port_data_parameters...));
-            auto insert_pair = m_hca_port_data_by_key_.insert(
-                make_pair<hca_port_key_t, std::unique_ptr<PortData>>(hca_port_key_t{port_info.m_port_key, port_info.m_first_plane},
-                                                                     nullptr));
+            auto insert_pair =
+                m_port_data_by_key_.insert(make_pair<port_key_t, std::unique_ptr<PortData>>(port_key_t{port_info.m_port_key}, nullptr));
             if (!insert_pair.second) {   // the key already exists
-                INFO("Replacing existing %s in m_port_data_by_key_ with key: %s", port_type_str, port_info.ToString().c_str());
+                INFO("Replacing existing %s in m_port_data_by_key_ with key: " U64H_FMT ":%hhu",
+                     port_type_str,
+                     port_info.m_port_key.GetKey(),
+                     port_info.m_port_key.GetPlane());
                 m_old_ports_.push_back(std::move(insert_pair.first->second));
                 m_old_ports_.back()->Disable();
             }
@@ -550,11 +630,6 @@ class Fabric
             throw;
         }
     }
-
-    lid_t AllocateMlid();
-    void FreeMlid(lid_t mlid);
-
-    friend SharableMlid;   // This enables SharableMlid to call private FreeMlid method
 };
 
 // Recognize the global variables
