@@ -1,6 +1,5 @@
 /*
- * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
- * Copyright (c) 2022-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -34,13 +33,11 @@
 
 #pragma once
 
-// std
-#include <cstdint>
 #include <unordered_set>
-// project
 #include "am_common.h"
 #include "dump_file.h"
 
+#include <cstdint>
 #include "fabric.h"
 #include "port_data.h"
 #include "reservation_manager.h"
@@ -56,9 +53,9 @@ class ReservationManager;
 class ParallelTreeFinder;
 
 using SetTreeEdgePtr = std::set<TreeEdge*>;
+using ListTreeIds = std::list<sharp_trees_t>;
 using JobSubTreeInfoVec = std::vector<JobSubTreeInfo>;
-using JobSubTreeInfoVecVec = std::vector<JobSubTreeInfoVec>;
-using MapAnToQuotaAllocInfo = std::map<const AggNodeFabricInfo*, AggNodeQuotaAllocInfo>;
+using MapAnToQuotaAllocInfo = std::map<AggNodeFabricInfo*, AggNodeQuotaAllocInfo>;
 using MapAnToQuotaAllocInfoInsertRes = std::pair<MapAnToQuotaAllocInfo::iterator, bool>;
 using VectorANFabricInfoAndANQuotaInfoPair = std::vector<std::pair<AggNodeFabricInfo const*, AggNodeQuotaAllocInfo const*>>;
 using MapPortKeyToQuota = std::map<string, sharp_quota>;
@@ -70,20 +67,15 @@ using VecVecPortData = std::vector<VecPortDataConstPtr>;
 using VecPlaneVecSetPortData = std::vector<VecSetPortData>;
 using VecPlaneVeRailVecPortData = std::vector<VecVecPortData>;
 
-enum class Confirmation
-{
-    NO,
-    YES,
-    ERROR_STATE,
-};
-
 struct AggNodeQuotaAllocInfo
 {
     Quota m_quota;
+    bool m_is_configured;
     ListTreeIds m_tree_ids;        // list of tree_ids on this node
     ListTreeIds m_root_tree_ids;   // list of tree_ids that are root on this node
-    bool m_prevent_lock{false};    // prevent lock on node (unless used by SAT)
-    Confirmation m_is_configured{Confirmation::NO};   // whether the node is configured on fabric (yes/no/error)
+    bool m_prevent_lock;           // prevent lock on node (unless used by SAT)
+
+    AggNodeQuotaAllocInfo() : m_is_configured(false), m_prevent_lock(true) { m_quota = Quota(); }
 };
 
 class SharpJob
@@ -96,7 +88,7 @@ class SharpJob
 
     uint8_t m_priority_;
 
-    bool m_multicast_enabled_;   // multicast of llt operation result
+    bool m_multicast_enabled_;
 
     // Tells whether we should update the relevant reservation when this job is deleted.
     // Reasons to avoid an update can be:
@@ -105,20 +97,18 @@ class SharpJob
     bool m_should_update_reservation_at_job_delete;
 
     string m_job_file_path;
-    string m_job_mc_trees_file_path_;
 
     // Job may contain hosts list or ports list
     SetHostInfoPtr m_hosts_;
     VecPlaneVecSetPortData m_ports_by_rail_;
+    uint8_t m_max_plane_;
 
     JobSubTreeInfoVec m_sub_tree_info_vec_;
-    JobSubTreeInfoVecVec m_mc_sub_tree_info_vec_;   // multicast trees structure
     MapAnToQuotaAllocInfo m_an_to_quota_alloc_info_;
 
-    uint8_t m_trees_number_;   // not include multicast trees
-    SetTreeIds m_tree_ids_;    // not include multicast trees
+    uint8_t m_trees_number_;
+    SetTreeIds m_tree_ids_;
     SetAggNodeFabricPtr m_agg_nodes_;
-    SetAggNodeFabricPtr m_mc_agg_nodes_;
 
     // DOTO remove m_num_channels_per_conn_ from SharpJob
     uint8_t m_num_channels_per_conn_;
@@ -174,7 +164,6 @@ class SharpJob
              file_utils::DumpFile* resource_alloc_dump_file);
 
     void SetJobFilePath(string job_file_path) { m_job_file_path = job_file_path; }
-    void SetJobMcTreesDumpFilePath(string& job_mc_trees_file_path) { m_job_mc_trees_file_path_ = job_mc_trees_file_path; }
     int AddHostInfo(const char* hostname);
 
     sharp_am_status AnalizeRailsInfo(const sharp_begin_job& begin_job_data);
@@ -219,24 +208,12 @@ class SharpJob
     JobSubTreeInfo* GetJobSubTreeInfoForTreeId(uint16_t tree_id);
 
     void ResizeSubTreeInfoVec(uint8_t num_trees) { m_sub_tree_info_vec_.resize(num_trees); }
-    void ResizeMulticastSubTreeInfoVecVec(uint8_t num_vec) { m_mc_sub_tree_info_vec_.resize(num_vec); }
-    void ResizeMulticastSubTreeInfoVec(uint8_t num_vec, uint8_t num_mc_trees) { m_mc_sub_tree_info_vec_[num_vec].resize(num_mc_trees); }
-    uint8_t GetMulticastSubTreeInfoSize(uint8_t num_vec)
-    {
-        return (GetMulticastSubTreeInfoVecVecSize() <= num_vec ? 0 : (uint8_t)m_mc_sub_tree_info_vec_[num_vec].size());
-    }
-    uint8_t GetMulticastSubTreeInfoVecVecSize() { return (uint8_t)m_mc_sub_tree_info_vec_.size(); }
 
     JobSubTreeInfo& GetSubTreeInfo(uint8_t tree_number) { return m_sub_tree_info_vec_[tree_number]; }
     const JobSubTreeInfo& GetSubTreeInfo(uint8_t tree_number) const { return m_sub_tree_info_vec_[tree_number]; }
     // get the next unused SubTreeInfo
     JobSubTreeInfo& GetSubTreeInfo() { return m_sub_tree_info_vec_[m_trees_number_]; }
 
-    JobSubTreeInfo& GetMulticastSubTreeInfo(uint8_t num_vec, uint8_t mc_tree_number)
-    {
-        return m_mc_sub_tree_info_vec_[num_vec][mc_tree_number];
-    }
-
     int ClearJobSubTreeInfo(sharp_trees_t tree_id);
 
     bool IsReproducible() const { return m_reproducible; }
@@ -250,17 +227,12 @@ class SharpJob
     uint8_t GetTreesNumber() const { return m_trees_number_; }
     const SetTreeIds& GetTreeIds() const { return m_tree_ids_; }
     const SetAggNodeFabricPtr& GetAggNodes() const { return m_agg_nodes_; }
-    const SetAggNodeFabricPtr& GetMulticastAggNodes() const { return m_mc_agg_nodes_; }
 
     u_int32_t GetHostNumber() const { return (u_int32_t)m_hosts_.size(); }
     const SetHostInfoPtr& GetHosts() { return m_hosts_; }
     const SetPortDataConstPtr& GetPorts(uint8_t rail, uint8_t plane_index) const { return m_ports_by_rail_[plane_index][rail]; }
     uint8_t GetNumberOfRails(uint8_t plane = 0) const { return m_ports_by_rail_[plane].size(); }
     uint8_t GetNumberOfPlanes() const { return m_ports_by_rail_.size(); }
-    uint8_t GetRailIndex(uint8_t tree_index) const { return tree_index % GetNumberOfRails(); }
-    uint8_t GetPlaneIndex(uint8_t tree_index) const { return tree_index / GetNumberOfRails(); }
-    uint8_t GetTreeIndex(uint8_t rail_index, uint8_t plane_index) const { return plane_index * GetNumberOfRails() + rail_index; }
-
     uint32_t GetPortsNumberForRail(uint8_t rail, uint8_t plane);
     uint32_t GetPortsNumberForAllRails();
     // uint32_t GetPortsNumberForAllRailsPlanes();
@@ -280,7 +252,6 @@ class SharpJob
     void SetJobInfoSendTimeStamp(bool job_info_send);
     void SetJobInfoReplyReceived(bool job_info_reply_received) { m_job_info_reply_received = job_info_reply_received; }
 
-    sharp_am_status FabricTreesConfig();
     sharp_am_status FabricQuotaConfig();
     void FabricJobResourceCleanup();
     void FabricJobResourceCleanupV2();
@@ -290,9 +261,6 @@ class SharpJob
     void CleanJobTreeConfigOnFabric();
     void DisableJobTreeNodes();
 
-    sharp_am_status FabricMulticastConfig(bool clear_config);
-    int CopyMulticastPersistentJobInfo(persistent_job_info* job_info);
-
     void IncreaseJobLoadOnAggNodes();
     void AddPendingTreeEdge(TreeEdge* p_tree_edge);
 
@@ -300,13 +268,12 @@ class SharpJob
 
     void GetSubTreesBFS(ListTreeNodePtr* sub_tree_nodes_list);
     void GetSubTreeNodesBFS(ListTreeNodePtr* sub_tree_nodes_list, struct JobSubTreeInfo& sub_tree_info);
-    void CreateJobInfoFiles(const string& persistent_path);
-    void DeleteJobInfoFiles(bool should_print = true);
+    void CreateJobInfoFile(const string& persistent_path);
+    void DeleteJobInfoFile();
 
     uint8_t GetNumChannelsPerConn() const { return m_num_channels_per_conn_; }
     void AddTreeId(sharp_trees_t tree_id);
     void AddAggNode(AggNodeFabricInfo* p_agg_node);
-    void AddMulticastAggNode(AggNodeFabricInfo* p_agg_node);
     int SetTreeRootForJob();
     int SetQuotaAllocForJob();
     bool IsExclusiveLockUsedByTree(sharp_trees_t tree_id);
@@ -329,32 +296,16 @@ class SharpJob
     static void FreeJobData(sharp_job_data& job_data);
     void PrepareJobTreesInfoMessage(sharp_job_trees_info& job_info);
     int ReconstructTrees();
-    int ReconstructMulticastTrees();
     int UpdateSubTreesInfo();
     void RestorePeerTreeIds(bool is_sat);
 
     void PrintResourceAllocationSummary();
     void CleanJobsDataRestoredDuringSeamlessRestart();
     void GetAnPortKeys(USetPortKeys& an_port_keys) const;
-    void GetAnPortKeysForMulticast(USetPortKeys& an_port_keys) const;
 
     void DumpJobTrees(file_utils::DumpFile*) const;
 
    private:
-    sharp_am_status AllocateJobMulticastResource(const JobResource& job_resource,
-                                                 const sharp_begin_job& begin_Job_data,
-                                                 uint8_t max_radix,
-                                                 uint8_t tree_index);
-    uint8_t GetNumOfRequiredMulticastTrees(uint8_t max_radix, uint16_t compute_ports_number, uint8_t max_mc_trees);
-
-    sharp_am_status AddComputesToMulticastTrees(const SetPortDataConstPtr& compute_ports, uint8_t tree_index);
-    sharp_am_status AddComputesToMulticastTrees(MultisetPortDataByPeerRange& range, uint8_t tree_index);
-
-    void AddJobToMulticastTree(JobSubTreeInfo& job_mc_sub_tree_info);
-    void RemoveJobFromMulticastTrees();
-    void RemoveJobFromMulticastTrees(JobSubTreeInfoVec& mc_sub_tree_info_vec);
-    void RemoveJobFromMulticastTree(JobSubTreeInfo& job_mc_sub_tree_info);
-
     VectorANFabricInfoAndANQuotaInfoPair GetANFabricInfoAndANQuotaInfoPairsSortedByPortRankAndGuid(
         const MapPortKeyToQuota& port_key_to_quota = {});
     void WriteResourcesAllocDetailsToFile(char const* const resource_msg_format,
@@ -376,15 +327,6 @@ class SharpJob
 
     void SetComputePorts();
 
-    void DeleteJobInfoFile(string& file_path, bool should_print = true);
-    void DumpJobMulticastTreesState();
-    void UpdateJobDataTreeMlids(sharp_tree& sharp_tree, uint32_t mc_tree);
-
-    void PrepareJobTreeInfoMessage(sharp_job_tree_info& job_tree_info, JobSubTreeInfo& job_tree, bool xdr_job);
-
-    int UpdateMulticastTables(SetAggNodeFabricPtr& mc_agg_nodes, bool clear_config);
-    void GetMulticastSubTreeNodesBFS(ListTreeNodePtr* sub_tree_nodes_list, JobSubTreeInfo& sub_tree_info, ListTreeNodePtr& nodes_queue);
-
     ////////                 Callback          ///////////
     // m_data1 = int operation_status;
     // m_data2 = *AggNode
