@@ -26,14 +26,14 @@
 #define ORTE_MAJOR_VERSION 4
 #define ORTE_MINOR_VERSION 1
 #define ORTE_RELEASE_VERSION 7
-#define ORTE_GREEK_VERSION "a1"
+#define ORTE_GREEK_VERSION "rc1"
 #define ORTE_WANT_REPO_REV @ORTE_WANT_REPO_REV@
-#define ORTE_REPO_REV "v4.1.5-115-g41ba5192d2"
+#define ORTE_REPO_REV "v4.1.5-176-g6d9519e4c3"
 #ifdef ORTE_VERSION
 /* If we included version.h, we want the real version, not the
    stripped (no-r number) verstion */
 #undef ORTE_VERSION
 #endif
-#define ORTE_VERSION "4.1.7a1"
+#define ORTE_VERSION "4.1.7rc1"
 
 #endif
