@@ -1,10 +1,13 @@
 /**
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
  * Copyright (c) 2015-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * See file LICENSE for terms.
  */
 
+#include <bits/posix1_lim.h>
 #include <coll_test.h>
 #include <ctype.h>
+#include <dlfcn.h>
 #include <sys/time.h>
 #include "api/sharp.h"
 
@@ -17,6 +20,17 @@
 #define SHARP_TBYTE (1ull << 40)
 #define SHARP_PBYTE (1ull << 50)
 
+#define LOAD_SYM(handle, symbol, funcptr)                                  \
+    do {                                                                   \
+        cast = (void**)&funcptr;                                           \
+        tmp = dlsym(handle, symbol);                                       \
+        if (tmp == NULL) {                                                 \
+            fprintf(stderr, "dlsym failed on %s - %s", symbol, dlerror()); \
+            goto teardown;                                                 \
+        }                                                                  \
+        *cast = tmp;                                                       \
+    } while (0)
+
 int client_id;
 int enable_sharp_coll = 1;
 int sharp_world_rank;
@@ -24,15 +38,14 @@ int sharp_world_size;
 int perf_test_iterations = 100;
 int perf_test_skips = 10;
 int perf_with_barrier = 1;
-int nbc_count = 512;
-int coll_root_rank = -1;
+int nbc_count = 16;
+int coll_root_rank = 0;
 int test_flat_ppn = 1;
 int sharp_test_group_type = COMM_TYPE_WORLD;
 int sharp_test_num_splits = 2; /* default odd even comm split */
 coll_sharp_component_t coll_sharp_component;
 char ib_dev_list[MAX_IB_DEVICES][16];
 int num_ib_devices;
-
 #ifdef MPI_SUPPORT
 extern MPI_Datatype sharp_test_mpi_min_max_datatype[SHARP_DTYPE_NULL][SHARP_DTYPE_NULL];
 #endif
@@ -73,27 +86,26 @@ static const char usage_string[] =
     "Usage:   sharp_coll_test[_mpi]  [OPTIONS]\n"
     "Options:\n"
     "-h, --help              Show this help message and exit\n"
-    "-B, --perf_with_barrier Sync Allreduce with Barrier in Allreduce collective, Default:1 \n"
-    "-c, --collectives       Comma separated list of collectives:[allreduce|iallreduce|reduce|ireduce|barrier|ibarrier|bcast|reduce_scatter|allgather|all|none]] to run.\n"
-    "                           Default: Allreduce\n"
-    "-d, --ib-dev            Use IB device <dev:port> (default first device found)\n"
-    "-j, --jobid             Explicit Job ID \n"
-    "-i, --iters             Number of iterations to run perf benchmark\n"
-    "-x, --skips             Number of warmup iterations to run perf benchmark\n"
-    "-M, --mem_type          Memory type(host,cuda,null) used in the communication buffers format: <src memtype>:<recv memtype>\n"
+    "-d, --ib-dev            Use IB device <dev:port> \n"
+    "-j, --jobid             Explicit Job ID (Default: Random ID) \n"
+    "-i, --iters             Number of iterations to run perf benchmark (Default: 100)\n"
+    "-x, --skips             Number of warmup iterations to run perf benchmark (Default: 10)\n"
+    "-M, --mem_type          Memory type(host,cuda,null) used in the communication buffers format: <src memtype>:<recv memtype> \n"
+    "                        (Dafault: host cpu memory on non-GPU setup. cuda memory for GPU setup\n"
 #ifdef MPI_SUPPORT
-    "-C, --iov_count         Number of entries in IOV list, if used. Default: 2 Max:SHARP_COLL_DATA_MAX_IOV \n"
-    "-D, --data_layout       Data layout (contig, iov) for sender and receiver side. Default: contig\n"
-    "-H, --host_alloc_type   Host memory allocation method ( hugetlb, hugethp(MADVISE Transparant Hugepage support), malloc)\n"
+    "-B, --perf_with_barrier Sync Allreduce with Barrier in Allreduce collective, Default:1 \n"
+    "-c, --collectives       Comma separated list of collectives:[allreduce|iallreduce|reduce|ireduce|barrier|ibarrier|bcast|reduce_scatter|allgather|all|none] to run.\n"
+    "                           Default: Allreduce\n"
+    "-H, --host_alloc_type   Host memory allocation method ( hugetlb, hugethp(MADVISE Transparent Hugepage support), malloc)\n"
     "-m, --mode              Test modes: <basic|complex|perf|all> . Default: perf\n"
     "-N, --nbc_count         Number of non-blocking operation posted before waiting for completion (max: 512)\n"
     "-p, --flat_ppn          Number of process per node default:1 \n"
     "-r, --root              Root for bcast, reduce operations \n"
     "-T, --datatype          Datatype of SHARP aggregation request(half_int, half_float, int, float, long, double) \n"
-    "-t, --group_type        Set specific group type(world(default), world-dup, world-rev, half, slipt:<n>, split-rev:<n>, split:ppn-comm, split:ppn-jobs) to test\n"
+    "-t, --group_type        Set specific group type(world(default), world-dup, world-rev, half, split:<n>, split-rev:<n>, split:ppn-comm, split:ppn-jobs) to test\n"
 #endif
     "-s, --size              Set the minimum and/or the maximum message size. format:[MIN:]MAX Default:<4:32M>\n"
-    "-f, --stepfactor        increment factor> multiplication factor between sizes. Default : 2";
+    "-f, --stepfactor        increment factor. multiplication factor between sizes. Set 1 to run test infinite loop. Default : 2";
 
 sharp_conf_t sharp_conf = {
     .rank = 0,
@@ -113,7 +125,12 @@ sharp_conf_t sharp_conf = {
     .r_mem_type = SHARP_MEM_TYPE_HOST,
     .datatype = SHARP_DTYPE_INT,
     .run_ppn_comm_parallel = 0,
-
+    .cuda_free = NULL,
+    .cuda_malloc = NULL,
+    .cuda_set_device = NULL,
+    .cuda_get_device_count = NULL,
+    .cuda_memcpy = NULL,
+    .cuda_lib = NULL,
 };
 int oob_bcast(void* comm_context, void* buf, int size, int root)
 {
@@ -226,9 +243,10 @@ static void sharp_component_init(struct coll_sharp_component_t* sharp)
 
 int coll_component_open()
 {
-    int ret;
+    int ret, d;
     struct sharp_coll_init_spec init_spec = {0};
     struct timeval tval;
+    char device_list[256] = "";
 
     coll_sharp_component.sharp_coll_context = NULL;
 
@@ -268,7 +286,6 @@ int coll_component_open()
     init_spec.oob_colls.gather = oob_gather;
     init_spec.oob_ctx = (void*)coll_sharp_component.mpi_comm;
     init_spec.config = sharp_coll_default_config;
-    init_spec.config.ib_dev_list = ib_dev_list[coll_sharp_component.node_local_rank % num_ib_devices];
 
     if (test_flat_ppn && coll_sharp_component.ppn > num_ib_devices &&
         (sharp_test_group_type != COMM_TYPE_PPN_COMM && sharp_test_group_type != COMM_TYPE_PPN_JOBS))
@@ -277,6 +294,29 @@ int coll_component_open()
         return -1;
     }
 
+    // split device list between all local ranks
+    if (num_ib_devices % coll_sharp_component.ppn != 0) {
+        fprintf(stderr, "Can't split device list equally between local ranks");
+        return -1;
+    }
+
+    int devices_per_ppn = num_ib_devices / coll_sharp_component.ppn;
+    d = 0;
+    while (1) {
+        strcat(device_list, ib_dev_list[coll_sharp_component.node_local_rank * devices_per_ppn + d]);
+        d++;
+        if (d == devices_per_ppn) {
+            break;
+        }
+        strcat(device_list, ",");
+    }
+
+    init_spec.config.ib_dev_list = device_list;
+
+    if (init_spec.world_size == 1) {
+        init_spec.config.flags |= SHARP_COLL_JOB_POLLING_KEEPALIVE;
+    }
+
 #if DEBUG
     fprintf(stdout,
             "[rank:%d, localrank:%d] Using IB device :%s world_local_rank:%d group_channel_idx:%d \n",
@@ -529,21 +569,21 @@ size_t sharp_test_string_quantity_prefix_value(char prefix)
  * Parssing memory units. valid syntax : <number>[b|kb|mb|gb].
  * The memory units ( 'b' , 'k', 'm', "g', 't') can be specificed in both upper and lower case.
  */
-int sharp_test_read_memunits(const void* str, size_t* dest, size_t low, size_t high)
+int sharp_test_read_memunits(const void* str, size_t* dest, ssize_t low, ssize_t high)
 {
     char units[3];
     int num_fields;
-    size_t value;
+    ssize_t value;
     size_t bytes;
 
     /* Special value: infinity */
     if (!strcasecmp(str, "inf")) {
-        *(size_t*)dest = ((size_t)-1);
+        *(size_t*)dest = SSIZE_MAX;
         return 0;
     }
 
     memset(units, 0, sizeof(units));
-    num_fields = sscanf(str, "%ld%c%c", &value, &units[0], &units[1]);
+    num_fields = sscanf(str, "%zd%c%c", &value, &units[0], &units[1]);
     if (num_fields == 1) {
         bytes = 1;
     } else if ((num_fields == 2) || (num_fields == 3)) {
@@ -580,7 +620,11 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
     while (res == PARSE_ARGS_OK) {
         option_index = 0;
         endptr = "\0";
+#ifdef MPI_SUPPORT
         opt = getopt_long(argc, argv, "ht:d:m:c:i:j:H:x:B:N:M:p:s:r:D:T:C:f:", long_options, &option_index);
+#else
+        opt = getopt_long(argc, argv, "hd:i:j:x:M:s:f:", long_options, &option_index);
+#endif
         if (opt == -1)   // no more args to parse
             break;
         switch (opt) {
@@ -611,7 +655,8 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                     } else if (strcmp(endptr, "none") == 0) {
                         sharp_conf->run_colls = 0;
                     } else if (strcmp(endptr, "all") == 0) {
-                        sharp_conf->run_colls |= TEST_ALLREDUCE | TEST_IALLREDUCE | TEST_BARRIER;
+                        sharp_conf->run_colls |=
+                            TEST_ALLREDUCE | TEST_IALLREDUCE | TEST_BARRIER | TEST_ALLGATHER | TEST_REDUCE_SCATTER | TEST_IBARRIER;
                     } else {
                         fprintf(stderr, "Invalid --collectives option\n");
                         res = INVALID_ARG;
@@ -635,6 +680,7 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                 } else if (strcmp(optarg, "iov") == 0) {
                     sharp_conf->sdata_layout = TEST_DATA_LAYOUT_IOV;
                     sharp_conf->rdata_layout = TEST_DATA_LAYOUT_IOV;
+                    res = INVALID_ARG;
                 } else {
                     fprintf(stderr, "Invalid --data_layout option\n");
                 }
@@ -732,16 +778,18 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                 break;
             case 'i':
                 tmp_id = strtol(optarg, &endptr, 10);
-                if (*endptr != '\0' || tmp_id < 0)
+                if (*endptr != '\0' || tmp_id < 0 || tmp_id > INT_MAX) {
+                    fprintf(stderr, "Invalid  number of interations specified\n");
                     res = INVALID_ARG;
-                else
+                } else
                     perf_test_iterations = tmp_id;
                 break;
             case 'f':
                 tmp_id = strtol(optarg, &endptr, 10);
-                if (*endptr != '\0' || tmp_id < 0)
+                if (*endptr != '\0' || tmp_id <= 0 || tmp_id > 2) {
+                    fprintf(stderr, "Invalid step factor value. acceptd values are 1(infinte) and 2\n");
                     res = INVALID_ARG;
-                else
+                } else
                     sharp_conf->stepfactor = tmp_id;
                 break;
             case 'j':
@@ -772,9 +820,10 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                 break;
             case 'x':
                 tmp_id = strtol(optarg, &endptr, 10);
-                if (*endptr != '\0' || tmp_id < 0)
+                if (*endptr != '\0' || tmp_id < 0 || tmp_id > INT_MAX) {
+                    fprintf(stderr, "Invalid  number of warmup interations specified\n");
                     res = INVALID_ARG;
-                else
+                } else
                     perf_test_skips = tmp_id;
                 break;
             case 'N':
@@ -802,6 +851,7 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                 saveptr = optarg;
                 endptr = strtok_r(optarg, ":", &saveptr);
                 if (endptr) {
+                    const char* p_invalid_value = "Invalid memory type value. Accepted values are: host, cuda, and null.\n";
                     if (0 == strcasecmp(endptr, "CUDA")) {
                         sharp_conf->s_mem_type = SHARP_MEM_TYPE_CUDA;
                     } else if (0 == strcasecmp(endptr, "HOST")) {
@@ -809,7 +859,7 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                     } else if (0 == strcasecmp(endptr, "NULL")) {
                         sharp_conf->s_mem_type = SHARP_MEM_TYPE_LAST;
                     } else {
-                        fprintf(stderr, "Invalid --mem_type option\n");
+                        fprintf(stderr, "%s", p_invalid_value);
                         res = INVALID_ARG;
                         break;
                     }
@@ -823,7 +873,7 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                         } else if (0 == strcasecmp(endptr, "NULL")) {
                             sharp_conf->r_mem_type = SHARP_MEM_TYPE_LAST;
                         } else {
-                            fprintf(stderr, "Invalid --mem_type option\n");
+                            fprintf(stderr, "%s", p_invalid_value);
                             res = INVALID_ARG;
                             break;
                         }
@@ -831,18 +881,30 @@ int parse_opts(int argc, char** argv, sharp_conf_t* sharp_conf)
                         sharp_conf->r_mem_type = sharp_conf->s_mem_type;
                     }
                 }
+                if (!sharp_conf->cuda_lib &&
+                    (sharp_conf->s_mem_type == SHARP_MEM_TYPE_CUDA || sharp_conf->r_mem_type == SHARP_MEM_TYPE_CUDA))
+                {
+                    fprintf(stderr, "CUDA runtime libs not found. CUDA memory not supported\n");
+                    res = INVALID_ARG;
+                }
                 break;
             case 's':
                 val1 = val2 = 0;
                 saveptr = optarg;
                 endptr = strtok_r(optarg, ":", &saveptr);
                 if (endptr) {
-                    sharp_test_read_memunits(endptr, &val1, 4, (size_t)-1);
+                    if (sharp_test_read_memunits(endptr, &val1, 4, SSIZE_MAX)) {
+                        res = INVALID_ARG;
+                        break;
+                    }
                     sharp_conf->min_message_size = sharp_coll_test_power_ceil(val1);
                     sharp_conf->max_message_size = sharp_coll_test_power_floor(val1);
                     endptr = strtok_r(NULL, ":", &saveptr);
                     if (endptr) {
-                        sharp_test_read_memunits(endptr, &val2, sharp_conf->min_message_size, (size_t)-1);
+                        if (sharp_test_read_memunits(endptr, &val2, sharp_conf->min_message_size, SSIZE_MAX)) {
+                            res = INVALID_ARG;
+                            break;
+                        }
                         sharp_conf->max_message_size = sharp_coll_test_power_floor(val2);
                     }
                 } else {
@@ -867,7 +929,7 @@ void print_parser_msg(int parse_res)
     else if (parse_res == INVALID_ARG)
         fprintf(stderr,
                 "ERROR. Either a non-existing command was specified, "
-                "or an option with a missing argument was given\n\n");
+                "or an option with a missing/invalid argument was given\n\n");
 }
 
 const char* sharp_get_host_name(void)
@@ -881,6 +943,35 @@ const char* sharp_get_host_name(void)
     return hostname;
 }
 
+static void coll_test_load_cuda(void)
+{
+    void* tmp;
+    void** cast;
+
+    sharp_conf.cuda_lib = dlopen("libcudart.so", RTLD_NOW);
+    if (!sharp_conf.cuda_lib) {
+        // fprintf(stderr, "Failed to open libcudart.so\n");
+        goto teardown;
+    }
+
+    /* Load the function pointers from the DL library image */
+    LOAD_SYM(sharp_conf.cuda_lib, "cudaGetDeviceCount", sharp_conf.cuda_get_device_count);
+    LOAD_SYM(sharp_conf.cuda_lib, "cudaSetDevice", sharp_conf.cuda_set_device);
+    LOAD_SYM(sharp_conf.cuda_lib, "cudaMalloc", sharp_conf.cuda_malloc);
+    LOAD_SYM(sharp_conf.cuda_lib, "cudaMemcpy", sharp_conf.cuda_memcpy);
+    LOAD_SYM(sharp_conf.cuda_lib, "cudaFree", sharp_conf.cuda_free);
+
+    sharp_conf.s_mem_type = SHARP_MEM_TYPE_CUDA;
+    sharp_conf.r_mem_type = SHARP_MEM_TYPE_CUDA;
+
+    return;
+teardown:
+    if (sharp_conf.cuda_lib != NULL)
+        dlclose(sharp_conf.cuda_lib);
+    sharp_conf.cuda_lib = NULL;
+    return;
+}
+
 int main(int argc, char** argv)
 {
     int rank, size, parse_res, mpi_rank, local_rank;
@@ -899,6 +990,8 @@ int main(int argc, char** argv)
     sharp_conf.size = sharp_world_rank = rank;
     sharp_conf.rank = sharp_world_size = size;
 
+    coll_test_load_cuda();
+
     parse_res = parse_opts(argc, argv, &sharp_conf);
     if (parse_res != PARSE_ARGS_OK) {
         if (rank == 0) {
@@ -941,11 +1034,10 @@ int main(int argc, char** argv)
 #endif
 
     if ((sharp_conf.s_mem_type == SHARP_MEM_TYPE_CUDA || sharp_conf.r_mem_type == SHARP_MEM_TYPE_CUDA)) {
-#if HAVE_CUDA
         int num_gpus, gpu_index, gpu_index_stride;
-        cudaError_t cerr;
-        cerr = cudaGetDeviceCount(&num_gpus);
-        if (cerr != cudaSuccess) {
+        int cerr;
+        cerr = sharp_conf.cuda_get_device_count(&num_gpus);
+        if (cerr != 0) {
             fprintf(stdout, "cudaGetDeviceCount failed\n");
             goto out;
         }
@@ -953,17 +1045,13 @@ int main(int argc, char** argv)
         gpu_index_stride = num_gpus / coll_sharp_component.ppn;
         gpu_index = (coll_sharp_component.node_local_rank * gpu_index_stride) % num_gpus;
 
-        cerr = cudaSetDevice(gpu_index);
-        if (cerr != cudaSuccess) {
+        cerr = sharp_conf.cuda_set_device(gpu_index);
+        if (cerr != 0) {
             fprintf(stdout, "cudaSetDevice failed\n");
             goto out;
         }
 #if DEBUG
         fprintf(stdout, "[Host:%s Rank:%d, GPU:%d] Testing with cuda buffers\n", coll_sharp_component.node_name, rank, gpu_index);
-#endif
-#else
-        fprintf(stderr, "CUDA support is not configured\n");
-        goto out;
 #endif
     }
 
